function varargout = pid_tuner(varargin)
% PID_TUNER M-file for the GUI of a PID type controller designer
%   This is part of the PID toolbox, where a rich group of PID tuning
%   formula are embeded in the interface.  One may select the controller
%   tuning algorithm and the controller can be designed and the closed-
%   loop system under the controller can be simulated.

%   Copyright (c) 2003
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @pid_tuner_OpeningFcn, ...
    'gui_OutputFcn',  @pid_tuner_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

function h_btnPlant_Callback(hObject, eventdata, handles)
uu=get(handles.h_btnPlant,'UserData');
if length(uu)==0
    plant_model4pid;
    set([handles.h_btnFOLPD,handles.h_btnModify],'Enable','on');
else
    uu
end

function h_btnModify_Callback(hObject, eventdata, handles)
plant_model4pid;
set(handles.h_txtFOLPD,'String','');
set(handles.h_btnFOLPD,'Enable','on');

function h_btnFOLPD_Callback(hObject, eventdata, handles)
key=get(handles.h_lstFOLPD,'Value');
try
    G=get(handles.h_btnPlant,'Userdata');
    L0=G.ioDelay; G.ioDelay=0;
    switch key                                 
        case 1
            [K,L,T]=getfod(G,1);     
        case 2
            [K,L,T]=getfod(G);
        case 3
            ff=msgbox({'Optimization process is going on. Please wait ...'},'Optimizing ...');
            Gr=opt_app(G,0,1,1);
            L=Gr.ioDelay;
            K=Gr.num{1}(end)/Gr.den{1}(end);
            T=Gr.den{1}(1)/Gr.den{1}(2);
            pause(0.0001);
            delete(ff);
    end
    L=L+L0;
    set(handles.h_btnFOLPD,'UserData',{K,L,T}); 
    str={'FOLPD model parameters:'; ['   K=',num2str(K),',  L=',num2str(L)]; ['   T=',num2str(T)]};
    set(handles.h_txtFOLPD,'String',str);
    set([handles.h_btnDesign,handles.h_btnSimulation,handles.h_btnDisplay],'Enable','on');   
catch
    errordlg({'FOLPD model failed'; lasterr},'Error');
end

function h_btnDesign_Callback(hObject, eventdata, handles)   
key1=get(handles.h_lstPIPID,'Value');
key2=get(handles.h_lstApply,'Value');
key3=get(handles.h_lstAlgorithm,'Value');
xx=get(handles.h_btnFOLPD,'UserData');
vars=[];
G=get(handles.h_btnPlant,'Userdata');
switch length(xx)
    case 0
        errordlg({'FOLPD model is not available,';'press ''Get FOLPD Model'' to get it.'},'Error')
    case 2            
    case 3
        K=xx{1}; L=xx{2}; T=xx{3};
end
switch key1
    case 1
        switch key2
            case 1
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_zn_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_hv_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ah_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ah_tune(key1,key2,vars);
                    case 5
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_cc_tune(key1,key2,vars);
                    case 6
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_wf_tune(key1,key2,vars);
                    case 7
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ml_tune(key1,key2,vars);
                    case 8
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_mcm_tune(key1,key2,vars);
                    case 9
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_cl_tune(key1,key2,vars);
                end
            case 2
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ml_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_sh_tune(G,key1,key2,vars);
                    case 3
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_hu_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_hv_tune(key1,key2,vars);
                    case 5
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ml_tune(key1,key2,vars);
                    case 6
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 7
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_ml_tune(key1,key2,vars);
                    case 8
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 9
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 10
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_chr_tune(key1,key2,vars);
                    case 11
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_chr_tune(key1,key2,vars);
                    case 12
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_sc_tune(key1,key2,vars);
                end
            case 3
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ro_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_hu_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 4
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_kl_tune(key1,key2,vars);
                    case 5
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ro_tune(key1,key2,vars);
                    case 6
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 7
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 8
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_chr_tune(key1,key2,vars);
                    case 9
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_chr_tune(key1,key2,vars);
                    case 10
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_sc_tune(key1,key2,vars);
                    case 11
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_sc_tune(key1,key2,vars);
                     case 12
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_han_tune(key1,key2,vars);
                end
            case 4
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ha_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_go_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_go_tune(key1,key2,vars);
                    case 4
                        set([handles.h_txtLam,handles.h_edtLam],'Visible','on');
                        lam=get(handles.h_edtLam,'String');
                        lambda=str2num(lam);
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ch_tune(lambda,key1,key2,vars);
                    case 5
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_as_tune(key1,key2,vars);
                    case 6
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_sch_tune(key1,key2,vars);
                    case 7
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_sch_tune(key1,key2,vars);
                    case 8
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_mca_tune(key1,key2,vars);
                    case 9
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_le_tune(G,key1,key2,vars);
                    case 10
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_kl_tune(key1,key2,vars);
                    case 11
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_han_tune(key1,key2,vars);
                    case 12
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_han_tune(key1,key2,vars);
                    case 13
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_han_tune(key1,key2,vars);
                    case 14
                        vars=[K L T 4];
                        [Kp,Ti,Td,key,lb]=pid_han_tune(key1,key2,vars);
                    case 15
                        vars=[K L T 5];
                        [Kp,Ti,Td,key,lb]=pid_han_tune(key1,key2,vars);
                    case 16
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ho_tune(key1,key2,vars);
                    case 17
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ho_tune(key1,key2,vars);
                    case 18
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_ho_tune(key1,key2,vars);
                    case 19
                        lambda=1;
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ta_tune(lambda,G,key1,key2,vars);
                    case 20
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_vl_tune(G,key1,key2,vars);
                    case 21
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_fw_tune(G,key1,key2,vars);
                    case 22
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_vl_tune(G,key1,key2,vars);
                    case 23
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_sm_tune(key1,key2,vars);
                    case 24
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_co_tune(key1,key2,vars);
                    case 25
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_cw_tune(key1,key2,vars);
                    case 26
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_co_tune(key1,key2,vars);
                    case 27
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_co_tune(key1,key2,vars);
                    case 28
                        vars=[K L T 4];
                        [Kp,Ti,Td,key,lb]=pid_co_tune(key1,key2,vars);
                    case 29
                        vars=[K L T 5];
                        [Kp,Ti,Td,key,lb]=pid_co_tune(key1,key2,vars);
                    case 30
                        vars=[K L T 6];
                        [Kp,Ti,Td,key,lb]=pid_co_tune(key1,key2,vars);
                    case 31
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ab_tune(key1,key2,vars);
                end
            case 5
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_br_tune(key1,key2,vars);
                    case 2
                        set([handles.h_txtLam,handles.h_edtLam],'Visible','on');
                        lam=get(handles.h_edtLam,'String');
                        lambda=str2num(lam);
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ri_tune(lambda,key1,key2,vars);
                    case 3
                        set([handles.h_txtLam,handles.h_edtLam],'Visible','on');
                        lam=get(handles.h_edtLam,'String');
                        lambda=str2num(lam);
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ri_tune(lambda,key1,key2,vars);
                    case 4
                        set([handles.h_txtLam,handles.h_edtLam],'Visible','on');
                        lam=get(handles.h_edtLam,'String');
                        lambda=str2num(lam);
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_chi_tune(lambda,key1,key2,vars);
                    case 5
                        set([handles.h_txtLam,handles.h_edtLam],'Visible','on');
                        lam=get(handles.h_edtLam,'String');
                        lambda=str2num(lam);
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_th_tune(lambda,key1,key2,vars);
                    case 6
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_fr_tune(key1,key2,vars);
                    case 7
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_og_tune(key1,key2,vars);
                    case 8
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_og_tune(key1,key2,vars);
                    case 9
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_og_tune(key1,key2,vars);
                    case 10
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_lee_tune(key1,key2,vars);
                end
            case 6
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_mcm_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_sh_tune(G,key1,key2,vars);
                    case 3
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_sh_tune(G,key1,key2,vars);
                    case 4
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_hw_tune(G,key1,key2,vars);
                    case 5
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_hw_tune(G,key1,key2,vars);
                    case 6
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_hw_tune(G,key1,key2,vars);
                    case 7
                        vars=[K L T 4];
                        [Kp,Ti,Td,key,lb]=pid_hw_tune(G,key1,key2,vars);
                    case 8
                        vars=[K L T 5];
                        [Kp,Ti,Td,key,lb]=pid_hw_tune(G,key1,key2,vars);
                    case 9
                        vars=[K L T 6];
                        [Kp,Ti,Td,key,lb]=pid_hw_tune(G,key1,key2,vars);
                    case 10 
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 11 
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 12
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_hwf_tune(G,key1,key2,vars);
                    case 13
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_hwf_tune(G,key1,key2,vars);
                    case 14
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_hwf_tune(G,key1,key2,vars);
                end
            case 7
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ah_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ah_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_ah_tune(key1,key2,vars);
                end
        end      
        x_result=[Kp,Ti];     
    case 2
        switch key2
            case 1
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_zn_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ah_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_pa_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_chr_tune(key1,key2,vars);
                    case 5
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_chr_tune(key1,key2,vars);
                    case 6
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_chr_tune(key1,key2,vars);
                    case 7
                        vars=[K L T 4];
                        [Kp,Ti,Td,key,lb]=pid_chr_tune(key1,key2,vars);
                    case 8
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ml_tune(key1,key2,vars);
                    case 9
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ml_tune(key1,key2,vars);
                    case 10
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_cc_tune(key1,key2,vars);
                end
            case 2
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ml_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_chh_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ml_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 5
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ml_tune(key1,key2,vars);
                    case 6
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 7
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                end
            case 3
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ro_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_wa_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_wa_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 5
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ro_tune(key1,key2,vars);
                    case 6
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_chh_tune(key1,key2,vars);
                    case 7
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_wa_tune(key1,key2,vars);
                    case 8
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 9
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                end
            case 4
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_sc_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_sc_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_sc_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_su_tune(key1,key2,vars);
                    case 5
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 6
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ab_tune(key1,key2,vars);
                    case 7
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ho_tune(key1,key2,vars);
                    case 8
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ho_tune(key1,key2,vars);
                end
            case 5
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_br_tune(key1,key2,vars);
                    case 2
                        set([handles.h_txtLam,handles.h_edtLam],'Visible','on');
                        lam=get(handles.h_edtLam,'String');
                        lambda=str2num(lam);
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ri_tune(lambda,key1,key2,vars);
                    case 3
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_fr_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_lee_tune(key1,key2,vars);
                end
            case 6
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_pe_tune(G,key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 3
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_za_tune(G,key1,key2,vars);
                    case 4
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_hwf_tune(G,key1,key2,vars);
                    case 5
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_hwf_tune(G,key1,key2,vars);
                    case 6
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_hwf_tune(G,key1,key2,vars);
                    case 7
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_mcm_tune(key1,key2,vars);
                    case 8
                        set([handles.h_txtLam,handles.h_edtLam],'Visible','on');
                        lam=get(handles.h_edtLam,'String');
                        lambda=str2num(lam);
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ta_tune(lambda,G,key1,key2,vars);
                    case 9
                        set([handles.h_txtLam,handles.h_edtLam],'Visible','on');
                        lam=get(handles.h_edtLam,'String');
                        lambda=str2num(lam);
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ta_tune(lambda,G,key1,key2,vars);
                end
            case 7
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ah_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ah_tune(key1,key2,vars);
                end
        end
        x_result=[Kp,Ti,Td,1]; 
    case 3  
        switch key2
            case 1
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_han_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ww_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_cl_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_cl_tune(key1,key2,vars);
                end
            case 2
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                end
            case 3
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 4];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                    case 5
                        vars=[K L T 5];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                    case 6
                        vars=[K L T 6];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                end
            case 4
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_tr_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 4
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 5
                        vars=[K L T 4];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 6
                        vars=[K L T 5];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 7
                        vars=[K L T 6];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 8
                        vars=[K L T 7];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 9
                        vars=[K L T 8];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 10
                        vars=[K L T 9];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 11
                        vars=[K L T 10];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                    case 12
                        vars=[K L T 11];
                        [Kp,Ti,Td,key,lb]=pid_ts_tune(key1,key2,vars);
                end
            case 5
                 set([handles.h_txtLam,handles.h_edtLam],'Visible','on');
                 lam=get(handles.h_edtLam,'String');
                 lambda=str2num(lam);
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_chi_tune(lambda,key1,key2,vars);
                    case 2
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_chi_tune(lambda,key1,key2,vars);
                end
            case 6
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_sh_tune(G,key1,key2,vars);
                    case 2
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_sh_tune(G,key1,key2,vars);
                end
            case 7
                switch key3
                    case 1
                        vars=[K L T 1];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                    case 2
                        vars=[K L T 2];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                    case 3
                        vars=[K L T 3];
                        [Kp,Ti,Td,key,lb]=pid_ks_tune(key1,key2,vars);
                end
        end
        x_result=[Kp,Ti,Td,2];    
end

set(handles.h_btnDesign,'UserData',{x_result,key1,key2,key3,key,lb})
h_btnDisplay_Callback(hObject, eventdata, handles);

function h_lstPIPID_Callback(hObject, eventdata, handles)
key1=get(handles.h_lstPIPID,'Value');         
key2=get(handles.h_lstApply,'Value');
str='';
switch key1
    case 1
        switch key2
            case 1
                str=str2mat('Ziegler & Nichols',...
                    'Hazebroek & Van der Waerden',...
                    'Astrom & Hagglund_ultimate cycle Ziegler-Nichols equivalent',...
                    'Astrom & Hagglund_20%overshoot',...
                    'Cohen & Coon',...
                    'Wolfe',...
                    'Murrill',...
                    'McMillan',...
                    'St. Clair');
            case 2
                str=str2mat('Minimum IAE (Murrill)',...
                    'Minimum IAE (Shinskey)',...
                    'Minimum IAE (Huang et al)',...
                    'Minimum ISE (Hazebroek & Van der Waerden)',...
                    'Minimum ISE (Murrill)',...
                    'Minimum ISE (Zhuang & Atherton)',...
                    'Minimum ITAE (Murrill)',...
                    'Minimum ISTSE (Zhuang & Atherton)',...
                    'Minimum ISTES (Zhuang & Atherton)',...
                    'Chien et al-regulator_0% overshoot',...
                    'Chien et al-regulator_20% overshoot',...
                    'Minimum IAE-regulator-Pemberton (Smith & Corripio)');
            case 3
                str=str2mat('Minimum IAE (Rovira et al)',...
                    'Minimum IAE (huang et al)',...
                    'Minimum ISE (Zhuang & Atherton)',...
                    'Minimum ISE (Khan & Lehman)',...
                    'Minimum ITAE (Rovira et al)',...
                    'Minimum ISTSE (Zhuang & Atherton)',...
                    'Minimum ISTES (Zhuang & Atherton)',...
                    'Chien et al-servo_0% over shoot',...
                    'Chien et al-servo_20% over shoot',...
                    'Minimum IAE-servo (Smith & Corripio)',...
                    '5% overshoot-servo (Smith & Corripio)',...
                    '5% overshoot-servo (Hang et al)');
            case 4
                str=str2mat('Haalman',...
                    'Regulator-Gorecki et al [real pole and max attainable multiplicity]',...
                    'Regulator-Gorecki et al [low freq.]',...
                    'Chiu et al',...
                    'Astrom et al',...
                    'Schneider [Closed loop response damping factor=1]',...
                    'Schneider [Closed loop response damping factor=0.6]',...
                    'McAnany',...
                    'Leva et al',...
                    'Khan & Lehman',...
                    'Hang et al [GM=10.5,PM=pi/6]',...
                    'Hang et al [GM=10.5,PM=pi/4]',...
                    'Hang et al [GM=10.5,PM=pi/3]',...
                    'Hang et al [GM=10.5,PM=3*pi/8]',...
                    'Hang et al [GM=10.5,PM=2*pi/5]',...
                    'Gain and margin (Ho et al) [Am=2.0,m=pi/4]',...
                    'Gain and margin (Ho et al) [Am=3.0,m=pi/3]',...
                    'Gain and margin (Ho et al) [Am=4.0,m=7*pi/18]',...
                    'Tan et al',...
                    'Symmetrical optimum principle (Voda & Landau)',...
                    'Friman & Waller',...
                    'Voda & Landau',...
                    'Smith',...
                    'Modulus optimum principle (Cox et al)',...
                    'Cluett & Wang [Closed loop time constant=4*L]',...
                    'Cluett & Wang [Closed loop time constant=2*L]',...
                    'Cluett & Wang [Closed loop time constant=1.33*L]',...
                    'Cluett & Wang [Closed loop time constant=L]',...
                    'Cluett & Wang [Closed loop time constant=0.8*L]',...
                    'Cluett & Wang [Closed loop time constant=0.67*L]',...
                    'Abbas');
            case 5
                str=str2mat('Brambilla et al',...
                    'Rivera et al',...
                    'Rivera et al alternative',...
                    'Chien',...
                    'Thomasson',...
                    'Fruehauf et al',...
                    'Ogawa [Closed loop peak gain=1.4,20% uncertainty ]',...
                    'Ogawa [Closed loop peak gain=1.4,33% uncertainty ]',...
                    'Ogawa [Closed loop peak gain=1.4,50% uncertainty ]',...
                    'Lee et al');
            case 6
                str=str2mat('McMillan',...
                    'Regulator-minimum IAE (Shinskey) [First Function ]',...
                    'Regulator-minimum IAE (Shinskey) [Second Function ]',...
                    'Regulator-nearly minimum IAE,ISE,ITAE (Hwang) [Decay ratio=0.15]',...
                    'Regulator-nearly minimum IAE,ISE,ITAE (Hwang) [Decay ratio=0.2]',...
                    'Regulator-nearly minimum IAE,ISE,ITAE (Hwang) [Decay ratio=0.25]',...
                    'Servo-small IAE (Hwang) [First Function]',... 
                    'Servo-small IAE (Hwang) [Second Function]',...
                    'Servo-small IAE (Hwang) [Third Function]',...
                    'Servo-minimum ISTSE (Zhuang & Atherton)',...
                    'Regulator-minimum ISTSE (Zhuang & Atherton)',...
                    'Servo-nearly minimum IAE & ITAE (Hwang & Fang)',...
                    'Regulator-nearly minimum IAE & ITAE (Hwang & Fang)',...
                    'Simultaneous Servo/regulator (Hwang & Fang)');
            case 7
                str=str2mat('Astrom & Hagglund [Ms=1.4-dominant pole design]',...
                    'Astrom & Hagglund [Ms=2.0-dominant pole design]',...
                    'Astrom & Hagglund-modified Ziegler-nichols');
        end
    case 2
        switch key2
            case 1 
                str=str2mat('Ziegler & Nichols',...
                    'Astrom & Hagglund',...
                    'Parr',...
                    'Chien et al-regulator_0% overshoot',...
                    'Chien et al-regulator_20% overshoot',...
                    'Chien et al-servo_0% overshoot',...
                    'Chien et al-servo_20% overshoot',...
                    'Three constrains method-Murrill',...
                    'Cohen & Coon',...
                    'Sain & Ozgen');
            case 2
                str=str2mat('Minimum IAE (Murrill)',...
                    'Modified minimum IAE (Cheng & Hung)',...
                    'Minimum ISE (Murrill)',...
                    'Minimum ISE (Zhuang & Atherton)',...
                    'Minimum ITAE (Murrill)',...
                    'Minimum ISTSE (Zhuang & Atherton)',...
                    'Minimum ISTES (Zhuang & Atherton)');
            case 3
                str=str2mat('Minimum IAE (Rovira et al)',...
                    'Minimum IAE (Wang et al)',...
                    'Minimum ISE (Wang et al)',...
                    'Minimum ISE (Zhuang & Atherton)',...
                    'Minimum ITAE (Rovira et al)',...
                    'Modified minimum ITAE (Cheng & Hung)',...
                    'Minimum ITAE (Wang et al)',...
                    'Minimum ISTSE (Zhuang & Atherton)',...
                    'Minimum ISTES (Zhuang & Atherton)');
            case 4
                str=str2mat('Regulator-min.IAE (Smith & Corripio)',...
                    'Servo-min.IAE (Smith & Corripio)',...
                    'Servo-5% overshoot (Smith & Corripio)',...
                    'Suyama',...
                    'Gain and phase margin (Zhuang & Atherton)',...
                    'Abbas',...
                    'Servo-min.ISE (Ho et al)',...
                    'Regulator-min.ISE (Ho et al)');
            case 5
                str=str2mat('Brambilla et al',...
                    'Rivera et al',...
                    'Fruehauf et al',...
                    'Lee et al');
            case 6
                str=str2mat('Regulator-minimum IAE (Pessen)',...
                    'Servo-minimum-ISTSE (Zhuang & Atherton)',...
                    'Regulator-minimum-ISTSE (Zhuang & Atherton)',...
                    'Servo-nearly minimum IAE ITAE (Hwang & Fang)',...
                    'Regulator-nearly minimum IAE ITAE (Hwang & Fang)',...
                    'Simultaneous Servo/regulator-nearly minimum IAE ITAE (Hwang & Fang)',...
                    'McMillan',...
                    'Tan et al [Am=2,m=pi/4]',...
                    'Tan et al [Arbitrary Am,m]');
            case 7
                str=str2mat('Astrom & Hagglund [max sensitivity=1.4]',...
                    'Astrom & Hagglund [max sensitivity=2.0]');
        end
    case 3
        switch key2
            case 1 
                str=str2mat('Hang et al',...
                    'Witt & Waggoner',...
                    'St.Clair ["aggressive" tuning]',...
                    'St.Clair ["conservative" tuning]');
            case 2
                str=str2mat('Minimum IAE (Kaya & Scheib)',...
                    'Minimum ISE (Kaya & Scheib)',...
                    'minimum ITAE (Kaya & Scheib)');
            case 3
                str=str2mat('Minimum IAE (Kaya & Scheib) [First Function]',...
                    'Minimum IAE (Kaya & Scheib) [Second Function]',...
                    'Minimum ISE (Kaya & Scheib) [First Function]',...
                    'Minimum ISE (Kaya & Scheib) [Second Function]',...
                    'minimum ITAE (Kaya & Scheib) [First Function]',...
                    'Minimum ITAE (Kaya & Scheib) [Second Function]');
            case 4
                str=str2mat('Tsang & Rad',...
                    'Tsang et al [=0.0]',...
                    'Tsang et al [=0.1]',...
                    'Tsang et al [=0.2]',...
                    'Tsang et al [=0.3]',...
                    'Tsang et al [=0.4]',...
                    'Tsang et al [=0.5]',...
                    'Tsang et al [=0.6]',...
                    'Tsang et al [=0.7]',...
                    'Tsang et al [=0.8]',...
                    'Tsang et al [=0.9]',...
                    'Tsang et al [=1.0]'); 
            case 5
                str=str2mat('Chien [First Function]',...
                    'Chien [Second Function]');
            case 6
                str=str2mat('Minimum IAE regulator (Shinskey) [First Funtion]',...
                    'Minimum IAE regulator (Shinskey) [Second Funtion]');
            case 7
                str=str2mat('Minimum IAE (Kaya & Scheib)',...
                    'Minimum ISE (Kaya & Scheib)',...
                    'minimum ITAE (Kaya & Scheib)');
        end
end
set(handles.h_lstAlgorithm,'String',str,'Value',1);

function h_lstPID_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

function h_lstPIPID_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

function pid_tuner_OpeningFcn(hObject, eventdata, handles, varargin)
handles.output = hObject;
guidata(hObject, handles);

function varargout = pid_tuner_OutputFcn(hObject, eventdata, handles)
varargout{1} = handles.output;

function h_lstApply_Callback(hObject, eventdata, handles)
h_lstPIPID_Callback(hObject, eventdata, handles);

function h_btnDisplay_Callback(hObject, eventdata, handles)
xx=get(handles.h_btnDesign,'UserData');
x_result=xx{1};key1=xx{2};key2=xx{3};key3=xx{4};key=xx{5};lb=xx{6};  
x_str1='';
x_str2='';
x_str3='';
x_str4='';
x_str5='';
switch length(x_result)
    case 0   
        x_str1='Failed';
    case 2
        x_str1='The PI controller is';
        if key2==7
            yy=get(handles.h_btnFOLPD,'UserData');
            L=yy{2};T=yy{3};
            t=L/(L+T);
            switch key3
                case 1
                    b=0.81*exp(0.73*t+1.9*t^2);
                    x_str3=sprintf('G_c(s)=%f(%f+1/(%fs))',x_result(1),b,x_result(2));
                case 2
                    b=0.44*exp(0.78*t-0.45*t^2);
                    x_str3=sprintf('G_c(s)=%f(%f+1/(%fs))',x_result(1),b,x_result(2));
                case 3
                    b=0.5;
                    x_str3=sprintf('G_c(s)=%f(%f+1/(%fs))',x_result(1),b,x_result(2));
            end
        else
            x_str3=sprintf('G_c(s)=%f(1+1/(%fs))',x_result(1),x_result(2));
        end
    case 4
        if x_result(4)==1
            x_str1='The PID controller is';
            if key2==7
                yy=get(handles.h_btnFOLPD,'UserData');
                L=yy{2};T=yy{3};
                t=L/(L+T); 
                switch key3
                    case 1
                        b=0.40*exp(0.18*t+2.8*t^2);
                        x_str3=sprintf('Gc_(s)=%f(%f+1/%fs+%fs)',x_result(1),b,x_result(2),x_result(3));
                    case 2
                        b=0.22*exp(0.65*t+0.051*t^2);
                        x_str3=sprintf('Gc_(s)=%f(%f+1/%fs+%fs)',x_result(1),b,x_result(2),x_result(3));
                end
            else
                x_str3=sprintf('Gc_(s)=%f(1+1/%fs+%fs)',x_result(1),x_result(2),x_result(3));
            end
        else
            x_str1='The Modified PID controller is';
            if key2==7      
                N=10;
                x_str3=sprintf('U(s)=%f(1+1/%fs)[R(s)-Y(s)(1+%fs)/(1+%fs/%6.2f)]',x_result(1),x_result(2),x_result(3),x_result(3),N);
            else
                if key2==4
                    if key3==1
                        N=5;
                    else
                        N=2.5;
                    end
                else
                    N=10;
                end
                x_str3=sprintf('Gc_(s)=%f(1+1/%fs)(1+%fs)/(1+%fs/%f)',x_result(1),x_result(2),x_result(3),x_result(3),N);
            end
        end
    otherwise
end
if key==2
    x_str5=sprintf('Warning: L/T>%f',lb);
elseif key==3
    x_str5=sprintf('Warning: L/T<%f',lb);
else
end
msgbox({x_str1; x_str2; x_str3; x_str2; x_str4;x_str5},'Controller information');

function h_btnSimulation_Callback(hObject, eventdata, handles)
G=get(handles.h_btnPlant,'UserData');
axes(handles.h_axMain);
set([handles.h_btnTRange,handles.h_axMain],'Visible','on');
xx=get(handles.h_btnDesign,'UserData');
key1=get(handles.h_lstPIPID,'Value');
key2=get(handles.h_lstApply,'Value');
key3=get(handles.h_lstAlgorithm,'Value');
Gd=[];
set([handles.h_txtLam,handles.h_edtLam],'Visible','off');
switch key1
    case 1
        if key2==7
            yy=get(handles.h_btnFOLPD,'UserData');
            L=yy{2};T=yy{3};
            t=L/(L+T);
            switch key3
                case 1
                    b=0.81*exp(0.73*t+1.9*t^2);
                    Gc=xx{1}(1)*(b+tf(1,[xx{1}(2) 0]));
                case 2
                    b=0.44*exp(0.78*t-0.45*t^2);
                    Gc=xx{1}(1)*(b+tf(1,[xx{1}(2) 0]));
                case 3
                    b=0.5;
                    Gc=xx{1}(1)*(b+tf(1,[xx{1}(2) 0]));
            end
        else
            Gc=xx{1}(1)*(1+tf(1,[xx{1}(2) 0]));
        end
    case 2
        if key2==7
            yy=get(handles.h_btnFOLPD,'UserData');
            L=yy{2};T=yy{3};
            t=L/(L+T); 
            switch key3
                case 1
                    b=0.40*exp(0.18*t+2.8*t^2);
                    Gc=xx{1}(1)*(b+tf(1,[xx{1}(2) 0])+tf([xx{1}(3),0],[xx{1}(3)/10,1]));
                case 2
                    b=0.22*exp(0.65*t+0.051*t^2);
                    Gc=xx{1}(1)*(b+tf(1,[xx{1}(2) 0])+tf([xx{1}(3),0],[xx{1}(3)/10,1]));
            end
        else
            Gc=xx{1}(1)*(1+tf(1,[xx{1}(2) 0])+tf([xx{1}(3),0],[xx{1}(3)/10,1]));
        end
    case 3
        if key2==7
            Gc=xx{1}(1)*(1+tf(1,[xx{1}(2) 0])); 
            Gd=Gc*tf([xx{1}(3),1],[xx{1}(3)/10,1]);
        else
            Gc=xx{1}(1)*(1+tf(1,[xx{1}(2) 0]))*tf([xx{1}(3),1],[xx{1}(3)/10,1]);
        end
end
tend=str2num(get(handles.h_edtTRange,'String')); 
if G.ioDelay==0 & ~(key1==3 & key2==7)
    step(feedback(G*Gc,1),tend); 
else
    assignin('base','num_p',G.num{1});
    assignin('base','den_p',G.den{1});
    assignin('base','num_c',Gc.num{1});
    assignin('base','den_c',Gc.den{1});
    assignin('base','Td',G.ioDelay);
    if length(Gd)>0
        assignin('base','num_d',Gd.num{1});
        assignin('base','den_d',Gd.den{1});
        [t,x,y]=sim('pid_simu_ind',[0,tend]);
    else
        [t,x,y]=sim('pid_simu',[0,tend]);
    end
    plot(t,y)
end

function h_btnTRange_Callback(hObject, eventdata, handles)
set([handles.h_edtTRange,handles.h_txtTRange],'Visible','on');

function h_edtTRange_Callback(hObject, eventdata, handles)
set([handles.h_edtTRange,handles.h_txtTRange],'Visible','off');
h_btnSimulation_Callback(hObject, eventdata, handles);

function h_btnHold_Callback(hObject, eventdata, handles)
key=get(handles.h_btnHold,'Value');
if key==0, hold off; else, hold on; end

%DO NOT edit 
function h_lstAlgorithm_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end
function h_lstAlgorithm_Callback(hObject, eventdata, handles)
function h_lstFOLPD_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end
function h_lstFOLPD_Callback(hObject, eventdata, handles)
function h_lstApply_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end
function h_edtLam_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end
function h_edtLam_Callback(hObject, eventdata, handles)
function h_edtTRange_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end
